// SPDX-License-Identifier: LGPL-3.0-or-later
// See Notices.txt for copyright information

use crate::{
    array::Array,
    enum_::Enum,
    int::{SInt, UInt},
    intern::{Intern, Interned},
};
use std::{fmt, hash::Hash};

#[derive(Copy, Clone, Hash, PartialEq, Eq, Debug)]
pub enum CanonicalType {
    UInt(UInt),
    SInt(SInt),
    Array(Interned<Array<CanonicalType>>),
    Enum(Enum),
}

pub trait Type: Copy + Hash + Eq + fmt::Debug + Send + Sync + 'static {
    fn canonical(&self) -> CanonicalType;
}

impl Type for CanonicalType {
    fn canonical(&self) -> CanonicalType {
        *self
    }
}

impl<T: Type> Type for Interned<T> {
    fn canonical(&self) -> CanonicalType {
        T::canonical(self)
    }
}

pub trait StaticType: Type {
    fn static_type() -> Self;
}

impl<T: StaticType> StaticType for Interned<T> {
    fn static_type() -> Self {
        Intern::intern_sized(T::static_type())
    }
}
